/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of signal processing functions
  \author Tony Ottosson, Thomas Eriksson, Pl Frenger, and Tobias Ringstrm

  1.5

  2002/10/07 12:31:10
*/
 
#ifndef __sigfun_h
#define __sigfun_h

#include "base/vec.h"
#include "base/transforms.h"

/*! \defgroup sigproc Signal Processing Functions
 */
//!@{

//! Cross correlation calculation
vec xcorr(const vec &x, const vec &y);

//! Auto correlation calculation
vec xcorr(const vec &x);

//! Covariance calculation.  Set is_zero_mean if m already has zero mean.
mat cov(const mat &m, bool is_zero_mean=false);

//vec cov(const vec &x, short order);

//! Power spectrum calculation using the Welch method and a Hanning window.
vec spectrum(const vec &v, int nfft=256, int noverlap=0);

//! Power spectrum calculation using the Welch method and the supplied window w.
vec spectrum(const vec &v, const vec &w, int noverlap=0);

//! Power spectrum calculation of a filter with transfer function a(z)
vec filter_spectrum(const vec &a, int nfft=256);

//! Power spectrum calculation of a filter with transfer function a(z)/b(z)
vec filter_spectrum(const vec &a, const vec &b, int nfft=256);

//!@}

#endif // __sigfun_h
